import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.BufferedReader;
import java.io.FileNotFoundException;

/**
* Read/save configuration data.
*/
public class Config
{
/** Configuration data */
protected static Properties data;

/** new properties */
protected static Properties newData;

/** Configuration file */
protected static File configfile;

/** Constructor */
public Config(String filename)
{
	configfile = new File(filename);
	data = new Properties();
	newData = new Properties();
}

/** Read configuration from the file */
public boolean load()
{
	BufferedReader r;
	boolean comment=false;
	boolean singlecomm=false;
	String line;

	try {

		r = new BufferedReader(new FileReader(configfile));
		while((line=r.readLine()) != null) {
			if(line.startsWith("//") ||
				 line.startsWith("#") ||
				 line.length()==0) {
				singlecomm=true;
				comment=true;
			}
			if(line.startsWith("/*")) {
				singlecomm=false;
				comment=true;
			}
			if( ! comment ) {
				StringTokenizer t=new StringTokenizer(line,"=");
				data.setProperty(t.nextToken(),t.nextToken());
			}
		
			if(line.endsWith("*/") || singlecomm) {
				comment = false;
			}
		}
	} catch(Exception e) {
		return false;
	}
	return true;
}

/** Return properties */
public static String getProperty(String key)
{
	return data.getProperty(key);
}

/** Set properties */
public static void setProperty( String key, String value )
{
	if( data.getProperty(key) == null ) {
		newData.setProperty(key,value);
	} else {
		data.setProperty(key,value);
	}
}

/** Get all the keys */
public static Enumeration propertyNames()
{
	return data.propertyNames();
}

/** Save the configuration */
public static boolean save()
{
	BufferedReader r;
	FileWriter f;
	FileReader f1;
	String line;
	String key;
	Enumeration e;
	boolean comment=false;
	boolean linecomm=false;
	int counter=0;
	
	try {

		File tFile = File.createTempFile("AAA",".conf", new File("./"));
		f = new FileWriter(tFile);

		f1 = new FileReader(configfile);
		r = new BufferedReader(f1);

		while((line=r.readLine()) != null) {

			if( line.startsWith("//") || line.startsWith("#") ||
	       		     line.length()==0) {
				comment=true;
				linecomm=true;
			}

			if(line.startsWith("/*")) {
				comment=true;
				linecomm=false;
			}

			if(!comment) {
				StringTokenizer t=
					new StringTokenizer(line,"=");
				key=t.nextToken();
				line=key+"="+data.getProperty(key);
				counter++;
			}
		
			if(line.endsWith("*/") || linecomm) {
				comment = false;
			}

			line += "\n";
			f.write( line );

		}

		e = newData.propertyNames();
		while( e.hasMoreElements() ) {
			key=(String) e.nextElement();
			line=key+"="+newData.getProperty(key)+"\n";
			f.write( line );
			data.setProperty(key,newData.getProperty(key));
		}

		newData.clear();

		f1.close();
		f.flush();
		f.close();

		configfile.delete();
		if(!tFile.renameTo(configfile)) {
			System.out.println("Error renaming file to " + configfile );
		}

	} catch(Exception x) {
		x.printStackTrace();
	}
	return true;
}
}

