import java.sql.*;
import java.util.ArrayList;
import java.util.StringTokenizer;

/**
* Allow generical query to be sent to a database
* trough JDBC interface.
*/
public class DBI
{

private Connection conn;
private Statement stm;
private ResultSet rs;
private ResultSetMetaData m;
private String url,login,password,driver,sql,catalog;
private ArrayList comlist;
private int lastcom;
private boolean finished;
private boolean executed;

/** list of known commands, add your commands to the end */
private static final String commands[]={
	"exit",
	"list",
	"exec",
	"help",
	"edit",
	"-1"
};

private static final int EXIT=0;
private static final int LIST=1;
private static final int EXEC=2;
private static final int HELP=3;
private static final int EDIT=4;
private static final int PREV=5;

/** The main program */
public static void main(String args[])
{
	DBI d=new DBI();
	if(!d.openDb())
		System.exit(0);
	d.getCommand();
}

/** constructor */
public DBI()
{
	comlist=new ArrayList();
	lastcom=0;
	executed=false;
	finished=false;
}

/** Open the database and initiate other things */
public boolean openDb()
{
	try {
		Config c=new Config("dbi.conf");
		c.load();
		url=c.getProperty("url");
		login=c.getProperty("login");
		password=c.getProperty("password");
		driver=c.getProperty("driver");
		catalog=c.getProperty("catalog");
		Class.forName(driver);
		conn=DriverManager.getConnection(url,login,password);
		if(catalog!=null)
			conn.setCatalog(catalog);
		return true;
	} catch(Exception e) {
		e.printStackTrace();
		return false;
	}
}

/** Execute a command (SQL) */
private void execute(String command)
{
	int columns=0;
	int i;

	if(executed)
		return;

	try {
		if(!comlist.contains(command)) {
			comlist.add(lastcom,command);
			lastcom++;
		}

		stm=conn.createStatement();
		stm.execute(command);
		i=stm.getUpdateCount();
		if(i>=0) {
			System.out.println(i+" rows processed");
		} else {
			while(true) {
				rs=stm.getResultSet();
				displayResult(rs);
				if(!stm.getMoreResults())
					break;
			}
		}
		stm.close();
	} catch(Exception e) {
		e.printStackTrace();
	}
}

/** Display resultset in the console */
private void displayResult(ResultSet rs)
{
	if( rs == null )
		return;

	int columns,i;
	try {
		m=rs.getMetaData();
		columns=m.getColumnCount();
		for(i=1;i<=columns;i++) {
			System.out.print(m.getColumnLabel(i).trim()+"\t");
		}
		System.out.print("\n==============\n");
		while(rs.next()) {
			for(i=1;i<=columns;i++) {
				String field=rs.getString(i);
				if( field == null )
					field = "NULL";
				System.out.print(field.trim()+"\t");
			}
			System.out.print("\n");
		}
	} catch(Exception e) {
		e.printStackTrace();
	}
	return;
}

/** Get a command from keyboard/console */
private void getCommand()
{
	int c=0;

	try {
		while(!finished) {
			executed=false;
			byte[] buffer=new byte[100];
			System.out.print("\nsql>");
			System.in.read(buffer);
			String command=new String(buffer);
			command=command.trim();

			checkRecall(command);
			checkKnown(command);
			execute(command);
		}
	} catch(Exception e) {
		return;
	}
}

/** Execute a known command */
private void checkKnown(String command)
{
	int c;

	for(c=0;c<commands.length;c++){
		if(command.compareToIgnoreCase(commands[c])==0){
			switch(c){
			case 0:
				finished=true;
				break;
			case 1:
				listCommand();
				break;
			case 2:
				break;
			case 3:
				showhelp();
				break;
			case 4:
				break;
			case 5:
				execold("-1");
				break;
			}
			executed=true;
		}
	}
	return;
}

/** Recall a previously recorded command */
private void checkRecall(String command)
{
	if(!command.startsWith("@")) {
		return;
	} else {
		command=command.substring(1);
		execold(command);
		executed=true;
		return;
	}
}

/** Re-execute an old command */
private void execold(String command)
{
	int i;
	if(lastcom==0)
		return;

	if(command.compareTo("-1")==0) {
		i=lastcom-1;
	} else {
		i=Integer.parseInt(command);
	}

	if(comlist.size()<=i) {
		System.out.println("no such command");
	} else {
		System.out.println(comlist.get(i));
		execute((String)comlist.get(i));
	}
}

/** List all the previously recorded commands */
private void listCommand()
{
	int i=0;
	for(i=0;i<lastcom;i++) {
		System.out.println(i+":"+comlist.get(i));
	}
}

/** Display a list of known commands */
private void showhelp()
{
	for(int i=0;i<commands.length;i++) {
		System.out.println(commands[i]);
	}
}

}
